#! /usr/bin/env python
# -*- coding: utf8 -*-

#
# Management program for Linux terminal server ALTSP, LTSP, NX
# Special for the ALT Linux School terminal
#
# Copyright (C) 2012,2013 Andrey Burbovskiy <xak-altsp@yandex.ru>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import with_statement
try:
    import pango
    import gtk, gobject
    import os, socket, threading
    import re, string, shlex, ConfigParser, gettext
    import datetime, time
except:
    print "Required python modules gtk,gobject,pango,os,socket,threading,re,string,shlex,ConfigParser,gettext,datetime,time,mimetypes"
    raise SystemExit
    
if gtk.pygtk_version < (2, 4):
    print "PyGtk 2.4 or later required"
    raise SystemExit

if ( os.getuid() == 0 ):
    print "Cannot be run under root"    
    raise SystemExit

try:
    import gtkvnc
except:
    print "python-module-gtkvnc not found. Program is run in reduced functionality."
    gtkvnc = False

# program modules
from config import *
from dialogs import *
from widget import *
from command import *
from window2 import *
from tree import *
from util import *
from timers import timers
from vnc import *

locale_path = os.path.expanduser("locale")
gettext.bindtextdomain('lts-user-control', locale_path)
gettext.textdomain('lts-user-control')
_ = gettext.gettext

###########################################################################################################################
############################################################################################################################
class Program():

    def __init__(self):
	
        self.cfg = cfg()
        self.cfg.read()
        self.cfg.read_icons()
        
	self.cfg.tooltips = gtk.Tooltips()
	    
	self.cfg.gtkvnc = False
	self.cfg.gtkvnc_bpp = False
	if ( gtkvnc ):
	    self.cfg.gtkvnc = True
	    for method in dir(gtkvnc.Display):
		if ( method == "set_depth" ):
		    self.cfg.gtkvnc_bpp = True
		    break
	
        self.cfg.vnc_active = False
        self.cfg.vnc_local_active = False
        self.colVnc = 0
        
	self.window = self.cfg.window
	self.window.set_title("LTS User Control")
	self.window.connect('check-resize', self.window_resize_event)
	self.window.connect("window-state-event", self.window_state_event)
	self.window.connect("event", self.window_event)
	self.window.connect("destroy", self.program_exit)
	self.mainUi()

    def callback(self, widget, data1=None ,data2=None, data3=None):
	# data2=row(path) thumbnails_vnc_event
	# data2=position scale, data3=event
	#print data1, data2, data3
	if (data1 == "screenshot_thumbnails" ):
	    user = data2[0]
	    server = data2[1]
	    vnc = data2[3]
	    time = datetime.datetime.now()
	    pix = vnc.get_pixbuf()
	    if ( os.path.exists(self.cfg.vncShotFolder) ):
    	    	pix.save(self.cfg.vncShotFolder+"/"+user+"_"+server+"_"+time.strftime("%d.%m.%Y_%H:%M:%S")+".png", "png", {})
    	    	self.cfg.status(_("The screenshot is saved to a file")+" "+user+"_"+server+"_"+time.strftime("%d.%m.%Y_%H:%M:%S")+".png")
    	    else:
    	    	self.cfg.status(_("The screenshot folder")+" "+_("not found")+" - "+self.cfg.vncShotFolder)

	if (data1 == "remove_tree_item" ):
	    remove_tree_item(self.cfg, self.treeView)
	if (data1 == "edit_tree_item" ):
	    userUi(self.cfg, self.window, self.tableMain2, self.treeView, "edit", get_selected_tree(self.treeView, "edit"))
	if (data1 == "create_standalone" ):
	    userUi(self.cfg, self.window, self.tableMain2, self.treeView, "standalone")
	if (data1 == "create_group" ):
	    userUi(self.cfg, self.window, self.tableMain2, self.treeView, "new_group")
	if (data1 == "create_server" ):
	    userUi(self.cfg, self.window, self.tableMain2, self.treeView, "server")
	    
	if (data1 == "client_info" ):
	    userUi(self.cfg, self.window, self.tableMain2, self.treeView, "client_info", get_selected_tree(self.treeView, "first"))

	if (data1 == "settings" ):
	    settings(self.cfg, self.window, self.tableMain2)

	if (data1 == "create_userList" ):
	    self.timers.timer_userList("restart")
	    	    
	if (data1 == "view" ):
	    if ( self.cfg.vnc_active or self.cfg.vnc_local_active ):
		return
	    if ( self.cfg.vncGtk == "y" and self.cfg.gtkvnc != False ):
		self.thumbnails_vnc("view")
	    else:
		create_vnc_viewer(self.cfg, get_selected_tree(self.treeView, "first"), "-viewonly")
		
	if (data1 == "control" ):
	    if ( self.cfg.vnc_active or self.cfg.vnc_local_active ):
		return
	    if ( self.cfg.vncGtk == "y" and self.cfg.gtkvnc != False ):
		self.thumbnails_vnc("control")
	    else:
		create_vnc_viewer(self.cfg, get_selected_tree(self.treeView, "first"), "")

	if (data1 == "close_gtkvnc" ):
	    self.panedTree.set_position(self.panedWindow.get_position()-self.cfg.phandle_size)
    	    
	if (data1 == "thumbnails" ):
	    if ( self.cfg.vnc_active or self.cfg.vnc_local_active ):
		return
	    # Xlib: sequence lost
	    # Xlib: unexpected async reply
	    # Без потока хуже...падает после произвольных кликов мышью во время открытия
	    #thread = thread_gfunc(self.cfg, False, True, self.thumbnails_vnc)
	    thread = thread_gfunc(self.cfg, True, True, self.thumbnails_vnc)
	    thread.start()
	    
	if (data1 == "vnc_servers" ):
	    self.vncServersUi.createUi()
	    
	if (data1 == "timers" ):
	    self.timersUi.createUi()
	    
	if (data1 == "process" ):
	    processUi(self.cfg, self.window, self.tableMain2, get_selected_tree(self.treeView))
	    
	if (data1 == "hwinfo" ):
	    hwinfoUi(self.cfg, self.window, self.tableMain2, get_selected_tree(self.treeView))

	if (data1 == "folder_user" ):
	    user_list = get_selected_tree(self.treeView, "first")
	    if ( user_list != [] ):
		folderUi(self.cfg, self.window, self.tableMain2, user_list)

	if (data1 == "log" ):
	    hwinfoUi(self.cfg, self.window, self.tableMain2, get_selected_tree(self.treeView), "log")
	    
	if (data1 == "run" ):
	    user_list = get_selected_tree(self.treeView)
	    if ( user_list != [] and self.entryBox.get_active_text() != "" ):
		if ( message_dialog(self.window, _("Run command")+" ?\n", user_list, True) == True ):
	    	    run_command(self.cfg, user_list, self.entryBox.get_active_text(), _("Run command"))
	        
	if (data1 == "send_message" ):
	    user_list = get_selected_tree(self.treeView)
	    if ( user_list != [] and self.entryBox.get_active_text() != "" ):
		if ( message_dialog(self.window, _("Send message")+" ?\n", user_list, True) == True ):
	    	    run_command(self.cfg, user_list, self.entryBox.get_active_text(), _("Send message"))

	if (data1 == "send_file" ):
	    user_list = get_selected_tree(self.treeView)
	    if ( user_list != [] and self.entryBox.get_active_text() != "" ):
		if ( message_dialog(self.window, _("Send file")+" ?\n", user_list, True) == True ):
	    	    send_file(self.cfg, user_list, self.entryBox.get_active_text())

	if (data1 == "console_server" ): 
	    user_list = get_selected_tree(self.treeView, "first")
	    if ( user_list != [] ):
		run_command(self.cfg, user_list, "", "console_server")
	    
	if (data1 == "console_host" ): 
	    user_list = get_selected_tree(self.treeView, "first")
	    if ( user_list != [] ):
		run_command(self.cfg, user_list, "", "console_host")
	    
	if (data1 == "wake_on_lan" ):
	    user_list = get_selected_tree(self.treeView)
	    if ( user_list != [] ):
		if ( message_dialog(self.window, _("Turn On")+" ?\n", user_list, True) == True ):
		    run_command(self.cfg, user_list, "", _("Turn On"))

	if (data1 == "logout" ):
	    user_list = get_selected_tree(self.treeView)
	    if ( user_list != [] ):
		if ( message_dialog(self.window, _("Logout")+" ?\n", user_list, True) == True ):
		    run_command(self.cfg, user_list, "", _("Logout"))

	if (data1 == "reboot" ):
	    user_list = get_selected_tree(self.treeView)
	    if ( user_list != [] ):
		if ( message_dialog(self.window, _("Reboot")+" ?\n", user_list, True) == True ):
		    run_command(self.cfg, user_list, "", _("Reboot"))

	if (data1 == "shutdown" ):
	    user_list = get_selected_tree(self.treeView)
	    if ( user_list != [] ):
		if ( message_dialog(self.window, _("Shutdown")+" ?\n", user_list, True) == True ):
		    run_command(self.cfg, user_list, "", _("Shutdown"))

	if (data1 == "lock" ):
	    user_list = get_selected_tree(self.treeView)
	    if ( user_list != [] ):
		if ( message_dialog(self.window, _("Lock screen")+" ?\n", user_list, True) == True ):
		    run_command(self.cfg, user_list, "", _("Lock screen"))

	if (data1 == "unlock" ):
	    user_list = get_selected_tree(self.treeView)
	    if ( user_list != [] ):
		if ( message_dialog(self.window, _("Unlock screen")+" ?\n", user_list, True) == True ):
		    run_command(self.cfg, user_list, "", _("Unlock screen"))
	    
	    
    def window_state_event(self, data=None, data2=None):
	pass

    def window_event(self, data=None, data2=None):
	# Реальный размер рабочего стола
	(self.cfg.screen_x, self.cfg.screen_y) = get_workspace()
	# Если активен локальный VNC
	if ( self.cfg.vncList[(0,)][58] != None and self.cfg.vncList[(0,)][58] != "" ):
	    if ( self.table_thumbnails.get_children() != [] ):
		self.panedTree.set_position(self.panedWindow.get_position()-self.cfg.phandle_size)
	    self.cfg.vnc_local_active = True
	else:
	    self.cfg.vnc_local_active = False
		
	if ( self.cfg.vnc_active or self.cfg.vnc_local_active ):
	    self.toolbarMain.get_children()[2].set_sensitive(False)
	    self.toolbarMain.get_children()[3].set_sensitive(False)
	    self.toolbarMain.get_children()[4].set_sensitive(False)
	    self.close_button_gtkvnc.set_sensitive(False)
	else:
	    self.toolbarMain.get_children()[2].set_sensitive(True)
	    self.toolbarMain.get_children()[3].set_sensitive(True)
	    self.toolbarMain.get_children()[4].set_sensitive(True)
	    self.close_button_gtkvnc.set_sensitive(True)

    def window_resize_event(self, data=None):
	# Размер окна программы
	(self.cfg.window_x, self.cfg.window_y) = self.window.get_size()
	
    	if ( self.tableMain2.get_children() == [] ): 
	    self.window.set_size_request(self.cfg.min_mainWindowX, self.cfg.min_mainWindowY)

	thumbnails = self.table_thumbnails.get_children()
	if ( thumbnails != [] ):
	    paned_x = self.panedWindow.get_position()
	    paned_tree_x = self.panedTree.get_position()
	    new_col_vnc = int((paned_x-paned_tree_x-40)/(self.cfg.vncThumbnailsX+10))
	    if ( new_col_vnc != self.colVnc and new_col_vnc != 0 ):
		col = -1
		row = 0
		for vbox in thumbnails:
		    col += 1
		    if ( col == new_col_vnc ):
		        row += 1
		        col = 0
		    if ( str(vbox).find("VBox") != -1 ):
    			self.table_thumbnails.move( vbox, col*(self.cfg.vncThumbnailsX+10)+15, row*(self.cfg.vncThumbnailsY+10)+10 )
    		self.colVnc = new_col_vnc
    			
    def paned_tree_event(self, data1=None, data2=None):
	# Ползунок разделения меньше treeX не передвинуть
	if ( self.panedTree.get_position() < self.cfg.treeX ):
    	    self.panedTree.set_position(self.cfg.treeX)
	# удалить миниатуры
	if ( self.panedTree.get_position() > self.panedWindow.get_position()-10 ):
	    self.close_gtkvnc()

	# Кнопка
	if ( self.panedTree.get_position() < self.panedWindow.get_position()-10 ):
	    self.close_button_gtkvnc.show_all()
	else:
	    self.close_button_gtkvnc.hide()


    def paned_window_event(self, data1=None, data2=None):
    	if ( self.tableMain2.get_children() != [] ): 
	    # Ползунок разделения не передвинуть
	    if ( self.panedWindow.get_position() < self.cfg.window_x-self.cfg.panedWindowX-self.cfg.phandle_size or\
		 self.panedWindow.get_position() > self.cfg.window_x-self.cfg.panedWindowX-self.cfg.phandle_size ):
    		self.panedWindow.set_position(self.cfg.window_x-self.cfg.panedWindowX-self.cfg.phandle_size)
    	else:
	    # Ползунок разделения не передвинуть
	    if ( self.panedWindow.get_position() < self.cfg.window_x-self.cfg.phandle_size ):
    	    	self.panedWindow.set_position(self.cfg.window_x-self.cfg.phandle_size)

	# Тащить за собой ползунок panedTree если нет миниатюр
	if( self.table_thumbnails.get_children() == [] ):
	    if ( self.panedTree.get_position() < self.panedWindow.get_position()-self.cfg.phandle_size ):
    	        self.panedTree.set_position(self.panedWindow.get_position()-self.cfg.phandle_size)


    def vnc_item(self, password, table, col, row, xsize, ysize, host, vncport, color, label, mode=None):
	# Создает VNC в контейнере VBox и возвращает его. Вызывается из thumbnails_vnc
	label = gtk.Label(label)
	label.show_all()
	label.modify_font(pango.FontDescription(self.cfg.fontThumbnails))
	vbox = gtk.VBox(False, 0)
	vbox.set_size_request(xsize, ysize+5)
    	table.put( vbox, col*(int(xsize)+10)+15, row*(int(ysize)+10)+10 )
	vnc = gtkvnc.Display()
	vbox.pack_start(vnc, expand=True, fill=True, padding=0)
	vbox.pack_start(label, expand=False, fill=False, padding=0)
	if ( mode != "control" ):
	    vnc.set_read_only(True)
	vnc.set_scaling(True)
	#vnc.set_pointer_local(True)
	#vnc.set_lossy_encoding(True)
	vnc.set_shared_flag(True)
	if ( color == "Full" ):
	    vnc.set_depth("VNC_DISPLAY_DEPTH_COLOR_FULL")
	elif ( color == "Medium" ):
	    vnc.set_depth("VNC_DISPLAY_DEPTH_COLOR_MEDIUM")
	elif ( color == "Low" ):
	    vnc.set_depth("VNC_DISPLAY_DEPTH_COLOR_LOW")
	elif ( color == "UltraLow" ):
	    vnc.set_depth("VNC_DISPLAY_DEPTH_COLOR_ULTRA_LOW")
	else:
	    vnc.set_depth("VNC_DISPLAY_DEPTH_COLOR_FULL")
	vnc.realize()
	vnc.set_credential(gtkvnc.CREDENTIAL_PASSWORD, password)
        vnc.open_host(host, vncport)
	vnc.connect("vnc-disconnected", self.vnc_item_disconnect, vbox)
	vnc.connect("vnc-initialized", self.vnc_item_init, vbox)
	return vbox, vnc

    def vnc_item_init(self, data, vbox):
	# Если show_all сделать отдельно, VBox будет пустым.
    	vbox.show_all()
    	
    def vnc_item_disconnect(self, data, vbox):
	pass

    def close_gtkvnc(self, data=None):
	for vbox in self.table_thumbnails.get_children():
	    #vbox.get_children()[0].close()
	    vbox.get_children()[0].destroy()
	    self.table_thumbnails.remove(vbox)

    def thumbnails_vnc_event(self, data, event):
	# Проверка кнопок мыши для миниатюр
        if event.button == 3:
    	    # user,server и ссылки на объекты хранятся в self.vnc_box
    	    # user,server должны быть уникальны в основном дереве
    	    for line in self.vnc_box:
    		user = line[0]
    		server = line[1]
    		vbox = line[2]
    		vnc_item = line[3]
    		if ( str(vbox).find(str(data)) != -1 ):
    	    	    # Поиск в дереве row(path) по user,server
    	    	    row = find_tree(self.cfg, self.cfg.userList, user, server)
    	    	    if ( row == False ):
    	    		return
    		    # Раскрыть и выделить
    		    self.treeView.scroll_to_cell(row, None, use_align=True, row_align=0.5, col_align=0.0)
    		    self.treeView.expand_to_path(row)
    		    self.treeSelection.unselect_all()
    		    self.treeSelection.select_path(row)
		    self.context_menu(event, line)

    def thumbnails_vnc(self, mode=None):
	# Функция заполнения vnc VBox-ами(миниатюрами)
	# Без модуля gtkvnc не работает
	if ( self.cfg.gtkvnc == False ):
	    return
	
	if ( mode ):
	    user_list = get_selected_tree(self.treeView, "first")
	else:
	    user_list = get_selected_tree(self.treeView)
	user_list = check_user_list(self.cfg, user_list, "vnc")
	if ( user_list == [] ):
	    return
	#
	self.cfg.vnc_active = True
	
	# Удалить все VBox из таблицы
	# Сам viewport(в paned window) останется прежнего размера :(
	# VBox-ы только в table,fixed и т.п., в treeview не получится, а жаль.
	self.close_gtkvnc()
	
	# Соответствие с основным списком
    	self.vnc_box = []
	col = -1
	row = 0

	# mode - встроенный в интерфейс VNC
	if ( mode ):
	    thumb_x = self.cfg.vncGtkX
	    thumb_y = self.cfg.vncGtkY
	else:
	    thumb_x = self.cfg.vncThumbnailsX
	    thumb_y = self.cfg.vncThumbnailsY
	
	# Ширину главного окна под размер одной миниатюры
	# Максимум ширина экрана
    	self.panedTree.set_position(self.cfg.treeX)
	# Декорации окна + отступ vncBox
	corr_x = 30+20+self.cfg.treeX
	corr_y = 220
	#
	real_x = self.panedWindow.get_position() - corr_x
	real_y = self.cfg.window_y - corr_y
	new_window_x = self.cfg.window_x
	new_window_y = self.cfg.window_y
	new_thumb_x = thumb_x
	new_thumb_y = thumb_y
	if ( real_x < thumb_x ):
	    corr = self.cfg.window_x - real_x
	    new_window_x = thumb_x+corr
	if ( real_y < thumb_y ):
	    new_window_y = thumb_y + corr_y
	
	if ( new_window_x != self.cfg.window_x or new_window_y != self.cfg.window_y ):
	    if ( new_window_x > self.cfg.screen_x ):
	        new_window_x = self.cfg.screen_x
	        new_thumb_x = self.cfg.screen_x - corr_x
	    if ( new_window_y > self.cfg.screen_y ):
	        new_window_y = self.cfg.screen_y
	        new_thumb_y = self.cfg.screen_y - corr_y

	    # Если размер VNC изменился
	    # Подгоняем под начальное соотношение сторон
	    if ( new_thumb_x != thumb_x or new_thumb_y != thumb_y ):
		ratio = float(thumb_x)/float(thumb_y)
		if ( float(new_thumb_x)/float(new_thumb_y) < ratio ):
    		    new_thumb_y = int(new_thumb_x/ratio)
		else:
    		    new_thumb_x = int(new_thumb_y*ratio)	
	    # Подгоняем окно под новый размер VNC
	    new_window_x = new_thumb_x + corr_x
	    new_window_y = new_thumb_y + corr_y
	    
	# Сколько по ширине
	self.colVnc = (real_x)/(int(new_thumb_x)+10)
	if ( self.colVnc < 1 ):
	    self.colVnc = 1
	
        for z in user_list:
    	    d = {}
	    for key, value in zip(self.cfg.z, z):
		d[key] = value

	    if ( mode ):
		(ip, vncport, vncviewer) = vnc_razrulit(self.cfg, z)
	    else:
		(ip, vncport, vncviewer) = vnc_razrulit(self.cfg, z, True)
		
    	    if ( vncport == "0" ):
    		continue
    	    col += 1
    	    (box,vnc_item) = self.vnc_item(d['vnc_pass'], self.table_thumbnails, col, row, new_thumb_x, new_thumb_y,\
    		 ip, vncport, d['vnc_gtk_color'], d['alias'], mode)
	    box.connect("button-press-event", self.thumbnails_vnc_event)
    	    self.vnc_box.append([d['user'],d['server'],box,vnc_item])
    	    if ( (col+1) == self.colVnc ): 
    		col = -1
    		row += 1
	#
	self.cfg.vnc_active = False
    	# Изменить окно
	if ( self.table_thumbnails.get_children() != [] ):
	    if ( self.tableMain2.get_children() != [] ):
		self.cfg.mainWindowLastX = new_window_x
		#close_window2(None, self.cfg, self.window, self.tableMain2)
	    self.window.resize(new_window_x, new_window_y)
	    self.close_button_gtkvnc.show_all()
	else:
	    if ( self.tableMain2.get_children() == [] ):
		self.window.resize(self.cfg.mainWindowX, self.cfg.mainWindowY)
    		self.panedTree.set_position(self.panedWindow.get_position()-self.cfg.phandle_size)
		
    def context_menu(self, event, line_vnc_box=None):
	# line_vnc_box объекты из self.vnc_box
	mouseMenu = gtk.Menu()
	
	if ( self.cfg.hideViewer == "n" ):
	    item = menu_image_button(self.cfg.pixbuf_action_viewer_16, _("Viewer"))
	    item.connect('activate', self.callback, "view")
    	    mouseMenu.append(item)

        if ( self.cfg.hideControl == "n" ):
	    item = menu_image_button(self.cfg.pixbuf_action_control_16, _("Control"))
	    item.connect('activate', self.callback, "control")
    	    mouseMenu.append(item)

	# снимок экрана только на миниатюре
	if ( line_vnc_box ):
	    item = menu_image_button(self.cfg.pixbuf_action_screenshot_16, _("Screenshot"))
	    item.connect('activate', self.callback, "screenshot_thumbnails", line_vnc_box)
    	    mouseMenu.append(item)

	if ( self.cfg.hideCommand == "n" ):
    	    item = gtk.SeparatorMenuItem() 
    	    mouseMenu.append(item)

	    item = menu_image_button(self.cfg.pixbuf_action_send_message_16, _("Send message"))
	    item.connect('activate', self.callback, "send_message")
    	    mouseMenu.append(item)

	    item = menu_image_button(self.cfg.pixbuf_action_run_16, _("Run command"))
	    item.connect('activate', self.callback, "run")
    	    mouseMenu.append(item)
        
	    item = menu_image_button(self.cfg.pixbuf_list_file_send_16,_("Send file"))
    	    item.connect("activate", self.callback, "send_file")
    	    mouseMenu.append(item)

	if ( self.cfg.hideUtil == "n" ):
    	    item = gtk.SeparatorMenuItem() 
    	    mouseMenu.append(item)
	
	    item = menu_image_button(self.cfg.pixbuf_home_16, _("Home folder"))
    	    item.connect("activate", self.callback, "folder_user")
    	    mouseMenu.append(item)

	    item = gtk.SeparatorMenuItem()
    	    mouseMenu.append(item)

    	    item = menu_image_button(self.cfg.pixbuf_console_server_16, _("Console"))
	    item.connect('activate', self.callback, "console_server")
    	    mouseMenu.append(item)
    	    
    	    item = menu_image_button(self.cfg.pixbuf_console_host_16, _("Console(host)"))
	    item.connect('activate', self.callback, "console_host")
    	    mouseMenu.append(item)
    	
    	    item = gtk.SeparatorMenuItem() 
    	    mouseMenu.append(item)

	    item = menu_image_button(self.cfg.pixbuf_process_16, _("View process"))
	    item.connect('activate', self.callback, "process")
    	    mouseMenu.append(item)

	    item = menu_image_button(self.cfg.pixbuf_hwinfo_16, _("Hardware Info"))
	    item.connect('activate', self.callback, "hwinfo")
    	    mouseMenu.append(item)

	    item = gtk.SeparatorMenuItem()
    	    mouseMenu.append(item)

	    item = menu_image_button(self.cfg.pixbuf_lock_16, _("Lock screen"))
	    item.connect('activate', self.callback, "lock")
    	    mouseMenu.append(item)

	    item = menu_image_button(self.cfg.pixbuf_unlock_16, _("Unlock screen"))
	    item.connect('activate', self.callback, "unlock")
    	    mouseMenu.append(item)
    	    
	if ( self.cfg.hideSystemUtil == "n" ):
    	    item = gtk.SeparatorMenuItem() 
    	    mouseMenu.append(item)

	    mouseMenuSystem = gtk.Menu()

	    item = menu_image_button(self.cfg.pixbuf_logout_16, _("Logout"))
	    item.connect('activate', self.callback, "logout")
    	    mouseMenuSystem.append(item)

	    item = menu_image_button(self.cfg.pixbuf_reboot_16, _("Reboot"))
	    item.connect('activate', self.callback, "reboot")
    	    mouseMenuSystem.append(item)

	    item = menu_image_button(self.cfg.pixbuf_turn_on_16, _("Turn On"))
	    item.connect('activate', self.callback, "wake_on_lan")
    	    mouseMenuSystem.append(item)

	    item = menu_image_button(self.cfg.pixbuf_shutdown_16, _("Shutdown"))
	    item.connect('activate', self.callback, "shutdown")
    	    mouseMenuSystem.append(item)
	
    	    item = menu_image_button(self.cfg.pixbuf_menu_system_16, "")
    	    item.set_submenu(mouseMenuSystem)
    	    mouseMenu.append(item)

        mouseMenu.popup(None, None, self.context_menu_pos, event.button, event.time, event)
	mouseMenu.show_all()

    def context_menu_pos(self, menu, event=None):
	x, y, mods = menu.get_root_window().get_pointer()
	ysize = len(menu.get_children())*16
	if ( y+ysize > self.cfg.screen_y ):
	    y = self.cfg.screen_y - ysize
	return (x, y, True)
	
    def status_changed(self, widget, event, data=None):
	# Устанавливает видимым последнее(нижнее) сообщение status()
        adj = self.swStatus.get_vadjustment()
        adj.set_value(adj.upper - adj.page_size)
	
    def treeViewTooltip(self, treeView, e, tooltips):
        try:
    	    (row,col,x,y) = treeView.get_path_at_pos( int(e.x), int(e.y) ) 
    	    model = treeView.get_model()
    	    user = model[row][0]
    	    host = model[row][1]
    	    tooltips.set_tip(treeView, user+","+host)
    	    tooltips.enable()
        except:
    	    pass

    def tree_button_press(self, treeView, event):
	mouse_pos = treeView.get_path_at_pos(int(event.x),int(event.y))
	# Снять выделение
	if ( not mouse_pos ):
	    treeView.get_selection().unselect_all()
	    return
	# multiple
	elif ( event.button == 3 or event.button == 1 ):
	    model, rows = treeView.get_selection().get_selected_rows()
	    if ( rows == [] ):
    		return
    	    if ( mouse_pos[0] in rows ):
    		treeView.get_selection().unselect_all()
		for row in rows:
    		    treeView.get_selection().select_path(row)
		# Отлючить выделение
		treeView.get_selection().set_select_function(lambda *ignore: False)
		# Обратно включить
		# Можно через button-release-event, но после popup-menu надо лишнее нажатие кнопки
		gobject.timeout_add(10, self.enable_tree_selection, self.treeView)
        # Действия
        if ( event.type == gtk.gdk._2BUTTON_PRESS ):
	    self.callback(None, "edit_tree_item")
	if ( event.button == 3 ):
	    self.context_menu(event)
		
    def enable_tree_selection(self, treeView):
	treeView.get_selection().set_select_function(lambda *ignore: True)

    def mainUi(self, data=None):


	self.vboxMain = gtk.VBox(False, 0)
	self.tableMain2 = gtk.Table(42, 28, True)
	self.panedWindow = gtk.HPaned()
	self.panedWindow.pack1(self.vboxMain, resize=False, shrink=True)
	self.panedWindow.pack2(self.tableMain2, resize=False, shrink=True)
        self.window.add(self.panedWindow)
        self.panedWindow.connect("notify::position", self.paned_window_event)

	#################
	# status
	#################
	self.swStatus = gtk.ScrolledWindow()
	self.swStatus.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
	self.textviewStatus = gtk.TextView(self.cfg.bufferStatus)
	self.textviewStatus.connect('size-allocate', self.status_changed)
	self.textviewStatus.set_property('editable', False)
	self.textviewStatus.set_cursor_visible(False)
	self.textviewStatus.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("light gray"))
	self.textviewStatus.modify_font(pango.FontDescription(self.cfg.fontStatus))
	self.swStatus.set_border_width(0)
	self.swStatus.add(self.textviewStatus)
	# Похож по цвету на gtk.StatusBar
	# без Viewport() цвет не изменить
	self.viewportStatus = gtk.Viewport()
	self.viewportStatus.add(self.swStatus)

	self.cfg.bufferStatus.insert(self.cfg.bufferStatus.get_end_iter(), "\n")
    	self.cfg.status(_("Starting"))

	self.treeView = gtk.TreeView(self.cfg.userList)
	self.treeView.set_rules_hint(True)
	self.treeView.set_enable_tree_lines(True)
	#self.treeView.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_VERTICAL)
	self.treeView.set_headers_visible(True)
	self.treeView.set_headers_clickable(False)
	self.treeView.modify_font(pango.FontDescription(self.cfg.fontTree))
	#color =  gtk.gdk.color_parse("black")
	#self.treeView.modify_base(gtk.STATE_SELECTED, color)
	self.treeView.connect("button-press-event", self.tree_button_press)
	
	d = {}
        for x in range(len(self.cfg.z)):
    	    d[self.cfg.z[x]] = int(x)
	
	i = -1
	for x in self.cfg.z:
	    i += 1
	    if ( i == d['alias'] ):
    		column = gtk.TreeViewColumn(_("Alias"))
		column.set_expand(True)
		column.set_spacing(3)
		
    		cell = gtk.CellRendererPixbuf()
		column.pack_start(cell, expand=False)
    		column.set_attributes(cell, pixbuf=50)

    		cell = gtk.CellRendererText()
    		cell.connect('edited', self.rename_group)
		column.pack_start(cell, expand=True)
    		column.set_attributes(cell, text=i)
    		for y in range(9):
    		    cell = gtk.CellRendererPixbuf()
    		    #if ( y == 0 ):
    		    #	cell.set_property("xalign", 0.0)
		    #	column.pack_start(cell, expand=True)
		    #else:
		    #	column.pack_start(cell, expand=False)
		    column.pack_start(cell, expand=False)
    		    column.set_attributes(cell, pixbuf=51+y)
	    else:
		cell = gtk.CellRendererText()
		if ( i == d['user'] ):
    		    column = gtk.TreeViewColumn(_("User"), cell, text=i)
		elif ( i == d['host'] ):
    		    column = gtk.TreeViewColumn(_("Host"), cell, text=i)
		elif ( i == d['ip'] ):
    		    column = gtk.TreeViewColumn(_("IP"), cell, text=i)
		elif ( i == d['server'] ):
    		    column = gtk.TreeViewColumn(_("Server"), cell, text=i)
		elif ( i == d['mac'] ):
    		    column = gtk.TreeViewColumn(_("MAC"), cell, text=i)
		elif ( i == d['start_time'] ):
    		    column = gtk.TreeViewColumn(_("Time"), cell, text=i)
    		else:
    		    column = gtk.TreeViewColumn(x, cell, text=i)
    		column.set_expand(True)
	    self.treeView.append_column(column)

	if ( self.cfg.treeShow == "full" ):
	    visible_columns = []
	else:
	    visible_columns = self.cfg.treeShow.split(",")

	for i in range(50):
    	    column = self.treeView.get_column(i)
    	    if ( visible_columns == [] ):
		column.set_visible(True)
		continue
    	    if ( str(i) in visible_columns):
		column.set_visible(True)
	    else:
		column.set_visible(False)
    	
    	column = self.treeView.get_column(0)
	cell = column.get_cell_renderers()[1]
	cell.set_property("visible", True)
	
	# tooltips
	#self.cfg.tooltips.set_tip(self.treeView, "")
	#self.treeView.connect("motion-notify-event", self.treeViewTooltip, self.cfg.tooltips)
	#self.treeView.set_events( gtk.gdk.POINTER_MOTION_MASK )

	# drag and drop
	if ( self.cfg.hideTreeDrag == "n" ):
	    TARGETS = [('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0), ('TEXT', 0, 0),]
	    self.treeView.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, TARGETS, gtk.gdk.ACTION_DEFAULT|gtk.gdk.ACTION_MOVE)
	    self.treeView.enable_model_drag_dest(TARGETS, gtk.gdk.ACTION_DEFAULT)
	    self.treeView.connect("drag_data_received", tree_drag_data_received, self.cfg)
	    self.treeView.connect("drag_data_get", tree_drag_data_get, self.cfg)	
	self.treeSelection = self.treeView.get_selection()
	self.treeSelection.set_mode(gtk.SELECTION_MULTIPLE)
	self.sw = gtk.ScrolledWindow()
	self.sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
	self.sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
	self.sw.add(self.treeView)
	
	######################
	# fast messages and commands
	######################
	self.entryBox = gtk.ComboBoxEntry(self.cfg.messageList, column=0)

	################
	# main toolbar
	################
	self.toolbarMain= gtk.Toolbar()
	self.toolbarMain.set_orientation(gtk.ORIENTATION_HORIZONTAL)
	self.toolbarMain.set_style(gtk.TOOLBAR_ICONS)
	self.toolbarMain.set_border_width(0)
	self.toolbarMain.set_tooltips(True)
	
	button = toolbar_button(self.cfg.pixbuf_action_refresh, self.cfg.tooltips, _("Refresh"))
	button.connect('clicked',self.callback, "create_userList")
	self.toolbarMain.insert(button,-1)

	item = gtk.SeparatorToolItem()
	self.toolbarMain.insert(item,-1)

	button = toolbar_button(self.cfg.pixbuf_action_viewer, self.cfg.tooltips, _("Viewer"))
	button.connect('clicked',self.callback, "view")
	if ( self.cfg.hideViewer == "n" ):
	    self.toolbarMain.insert(button,-1)
	
	button = toolbar_button(self.cfg.pixbuf_action_control, self.cfg.tooltips, _("Control"))
	button.connect('clicked',self.callback, "control")
	if ( self.cfg.hideControl == "n" ):
	    self.toolbarMain.insert(button,-1)

	button = toolbar_button(self.cfg.pixbuf_action_thumbnails, self.cfg.tooltips, _("Thumbnails"))
	button.connect('clicked',self.callback, "thumbnails")
	self.toolbarMain.insert(button,-1)

	item = gtk.SeparatorToolItem()
	self.toolbarMain.insert(item,-1)

	button = toolbar_button(self.cfg.pixbuf_action_vnc_servers, self.cfg.tooltips, _("VNC servers and clients"))
	button.connect('clicked',self.callback, "vnc_servers")
	if ( self.cfg.hideVncServers == "n" ):
	    self.toolbarMain.insert(button,-1)

	button = toolbar_button(self.cfg.pixbuf_action_timers, self.cfg.tooltips, _("Timers"))
	button.connect('clicked',self.callback, "timers")
	if ( self.cfg.hideTimer == "n" ):
	    self.toolbarMain.insert(button,-1)

	button = toolbar_button(self.cfg.pixbuf_action_user_info, self.cfg.tooltips, _("Client information"))
	button.connect('clicked',self.callback, "client_info")
	if ( self.cfg.hideClientInfo == "n" ):
	    self.toolbarMain.insert(button,-1)

	item = gtk.SeparatorToolItem()
	item.set_draw(False) 
	item.set_expand(gtk.EXPAND)
	self.toolbarMain.insert(item,-1)

	# toolbar menu util
	button = menu_tool_button(self.cfg.pixbuf_menu_util, self.cfg.tooltips)
	if ( self.cfg.hideUtil == "n" ):
	    self.toolbarMain.insert(button,-1)

	item = menu_image_button(self.cfg.pixbuf_home, _("Home folder"))
    	item.connect("activate", self.callback, "folder_user")
    	button.append(item)

	item = gtk.SeparatorMenuItem()
    	button.append(item)

    	item = menu_image_button(self.cfg.pixbuf_console_server, _("Console"))
	item.connect('activate', self.callback, "console_server")
    	button.append(item)
    	    
    	item = menu_image_button(self.cfg.pixbuf_console_host, _("Console(host)"))
	item.connect('activate', self.callback, "console_host")
    	button.append(item)
    	
	item = gtk.SeparatorMenuItem()
    	button.append(item)

	item = menu_image_button(self.cfg.pixbuf_process, _("View process"))
	item.connect('activate', self.callback, "process")
    	button.append(item)

	item = menu_image_button(self.cfg.pixbuf_hwinfo, _("Hardware Info"))
	item.connect('activate', self.callback, "hwinfo")
    	button.append(item)

	item = gtk.SeparatorMenuItem()
    	button.append(item)

	item = menu_image_button(self.cfg.pixbuf_lock, _("Lock screen"))
	item.connect('activate', self.callback, "lock")
    	button.append(item)

	item = menu_image_button(self.cfg.pixbuf_unlock, _("Unlock screen"))
	item.connect('activate', self.callback, "unlock")
    	button.append(item)

	# toolbar menu logout
	button = menu_tool_button(self.cfg.pixbuf_menu_system, self.cfg.tooltips)
	if ( self.cfg.hideSystemUtil == "n" ):
	    self.toolbarMain.insert(button,-1)

	item = menu_image_button(self.cfg.pixbuf_logout, _("Logout"))
	item.connect('activate', self.callback, "logout")
    	button.append(item)

	item = menu_image_button(self.cfg.pixbuf_reboot, _("Reboot"))
	item.connect('activate', self.callback, "reboot")
    	button.append(item)

	item = menu_image_button(self.cfg.pixbuf_turn_on, _("Turn On"))
	item.connect('activate', self.callback, "wake_on_lan")
    	button.append(item)

	item = menu_image_button(self.cfg.pixbuf_shutdown, _("Shutdown"))
	item.connect('activate', self.callback, "shutdown")
    	button.append(item)
	
	##################
	# tree toolbar
	#################
	self.toolbarTree = gtk.Toolbar()
	self.toolbarTree.set_orientation(gtk.ORIENTATION_HORIZONTAL)
	self.toolbarTree.set_style(gtk.TOOLBAR_ICONS)
	self.toolbarTree.set_border_width(0)
	self.toolbarTree.set_tooltips(True)

	# toolbar menu
	button = menu_tool_button(self.cfg.pixbuf_list_add_16, self.cfg.tooltips)
	button.show_all()
	
	self.toolbarTree.insert(button,-1)
	if ( self.cfg.hideTreeAddRemove == "y" ):
	    button.set_sensitive(gtk.FALSE)

	item = menu_image_button(self.cfg.pixbuf_list_add_16, _("Add standalone client"))
	item.connect('activate', self.callback, "create_standalone")
    	button.append(item)

	item = menu_image_button(self.cfg.pixbuf_list_add_16, _("Add group"))
	item.connect('activate', self.callback, "create_group")
    	button.append(item)

	item = menu_image_button(self.cfg.pixbuf_list_add_16, _("Add server"))
	item.connect('activate', self.callback, "create_server")
    	button.append(item)

	if ( self.cfg.hideTreeAddRemove == "n" ):
	    button = toolbar_button(self.cfg.pixbuf_list_remove_16, self.cfg.tooltips, _("Remove"))
	    button.connect('clicked', self.callback, "remove_tree_item")
	    self.toolbarTree.insert(button,-1)

	    button = toolbar_button(self.cfg.pixbuf_list_edit_16, self.cfg.tooltips, _("Edit"))
	    button.connect('clicked', self.callback, "edit_tree_item")
	    self.toolbarTree.insert(button,-1)

	item = gtk.SeparatorToolItem()
	item.set_draw(False) 
	item.set_expand(gtk.EXPAND)
	self.toolbarTree.insert(item,-1)

	if ( self.cfg.hideCommand == "n" ):
	    # menu
	    button = toolbar_button(self.cfg.pixbuf_action_send_message_16, self.cfg.tooltips, _("Send message"))
	    button.connect('clicked', self.callback, "send_message")
	    self.toolbarTree.insert(button,-1)

	    button = toolbar_button(self.cfg.pixbuf_action_run_16, self.cfg.tooltips, _("Run command"))
	    button.connect('clicked', self.callback, "run")
	    self.toolbarTree.insert(button,-1)

	    button = toolbar_button(self.cfg.pixbuf_list_file_send_16, self.cfg.tooltips, _("Send file"))
    	    button.connect("clicked", self.callback, "send_file")
	    self.toolbarTree.insert(button,-1)

	self.close_button_gtkvnc = toolbar_button(self.cfg.pixbuf_action_close_16, self.cfg.tooltips, _("Close")+" VNC")
	self.close_button_gtkvnc.connect('clicked',self.callback, "close_gtkvnc")
	self.toolbarTree.insert(self.close_button_gtkvnc,-1)

        ############################
        # paned tree
        ############################
	self.panedTree = gtk.HPaned()
	self.panedTree.add1(self.sw)

	self.swVnc = gtk.ScrolledWindow()
	self.swVnc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
	self.panedTree.add2(self.swVnc)
	
	self.table_thumbnails = gtk.Fixed()
	self.swVnc.add_with_viewport(self.table_thumbnails)
	self.panedTree.set_position(self.cfg.window_x-self.cfg.phandle_size)
        self.panedTree.connect("notify::position", self.paned_tree_event)

	##########
	# menu
	##########
	self.menuBar = gtk.MenuBar()
	self.menuFile = gtk.Menu()
	self.menuConfig = gtk.Menu()
	self.menuHelp = gtk.Menu()
	self.itemFile = gtk.MenuItem(_("File"))
	self.itemHelp = gtk.MenuItem(_("Help"))
	self.itemFile.set_submenu(self.menuFile)
	self.itemHelp.set_submenu(self.menuHelp)
                                     
	self.itemExit = gtk.MenuItem(_("Quit"))
	self.itemExit.connect("activate", self.program_exit)
        self.menuFile.append(self.itemExit)
        
                                                             
	self.itemAbout = gtk.MenuItem(_("About"))
	self.itemAbout.connect("activate", about_dialog)
        self.menuHelp.append(self.itemAbout)
                                                             
        self.menuBar.append(self.itemFile)

	self.itemConfig = gtk.MenuItem(_("Settings"))
	self.itemConfig.set_submenu(self.menuConfig)
	self.itemSettings = gtk.MenuItem(_("Settings"))
	self.itemSettings.connect("activate", self.callback, "settings")

	self.itemLog = gtk.MenuItem(_("Log"))
	self.itemLog.connect("activate", self.callback, "log")

	#self.itemLanguage = gtk.MenuItem(_("Language"))
	#self.itemLanguage.connect("activate", self.createLanguageWindow)
    	#self.menuConfig.append(self.itemLanguage)

        if ( self.cfg.hideSetting == "n" ):
	    self.menuConfig.append(self.itemSettings)
	    self.menuConfig.append(self.itemLog)
            self.menuBar.append(self.itemConfig)
        self.menuBar.append(self.itemHelp)
    
    	self.fileButton = image_button(self.cfg.pixbuf_list_file_add_16, None, self.cfg.tooltips, _("Select the file"))
    	self.fileButton.connect("clicked", file_chooser_dialog, self.entryBox, _("Select the file"))
	
        ##########################################
	# attach
	##########################################

	hbox = gtk.HBox(False, 0)
        hbox.pack_start(self.entryBox, expand=True, fill=True, padding=0)
        hbox.pack_start(self.fileButton, expand=False, fill=False, padding=0)
	
        self.vboxMain.pack_start(self.menuBar, expand=False, fill=False, padding=0)
        self.vboxMain.pack_start(self.toolbarMain, expand=False, fill=False, padding=0)
        self.vboxMain.pack_start(self.panedTree, expand=True, fill=True, padding=0)
        if ( self.cfg.hideCommand == "n" and self.cfg.hideTreeAddRemove == "n" ):
    	    self.vboxMain.pack_start(self.toolbarTree, expand=False, fill=False, padding=0)
	if ( self.cfg.hideCommand == "n" ):
    	    self.vboxMain.pack_start(hbox, expand=False, fill=False, padding=0)
    	    #self.vboxMain.pack_start(self.entryBox, expand=False, fill=False, padding=0)
        self.vboxMain.pack_start(self.viewportStatus, expand=False, fill=False, padding=0)

	# таймеры
	self.timers_start()
	
	self.window.show_all()
	self.close_button_gtkvnc.hide()

    def timers_start(self):
	self.timers = timers(self.cfg, self.treeView)
	# Формирование userList
	self.timers.timer_userList("start")
	#
        create_timersList(self.cfg)
        create_vncList(self.cfg)
	# 
	self.vncServersUi = vncServersUi(self.cfg, self.window, self.tableMain2, self.treeView)
	self.timersUi = timersUi(self.cfg, self.window, self.tableMain2, self.treeView, self.timers)
	#
	self.timers.start()
    
    def rename_group(self, cell, path, new_text):
	cell.set_property('editable', False)
	self.treeView.get_selection().select_path(path)
	model, rows = self.treeView.get_selection().get_selected_rows()
	if ( new_text == model[rows[0]][0] ):
    	    return False
	self.cfg.userList[path][0] = new_text
	save_userList(self.cfg)

    def program_exit(self, data=None):
	self.close_gtkvnc()
	self.timers.stop()
    	self.cfg.status(_("Quit"))
	umount_point(self.cfg)
	gtk.main_quit()
        
############################################################################################################################
if __name__ == "__main__":
    gobject.threads_init()
    gtk.gdk.threads_init()
    base = Program()
    gtk.gdk.threads_enter()
    gtk.main()
    gtk.gdk.threads_leave()

